--[[
Author: Ayantir
Filename: GuildChat.lua
Version: 5.5
]]--

--Libraries
local LC = LibStub('libChat-1.0')
local LAM2 = LibStub("LibAddonMenu-2.0")

-- Main variable
GuildChat = {}
GuildChat.db = {}

local characterNameMenuOptions = 
{
	[1] = "@account",
	[2] = "character",
	[3] = "character@account",
}

local colorOptions = 
{
	[1] = "Alliance",
	[2] = "Class",
	[3] = "None",
	[4] = "Custom1",
	[5] = "Custom2",
	[6] = "Custom3",
	[7] = "Custom4",
}

local Addon =
{
    Name = "GuildChat",
    NameSpaced = "Guild Chat",
    Author = "CrazyDutchGuy & Wobin & Ayantir",
    Version = "5.5",
    Defaults = {
		characterName = characterNameMenuOptions[1],
		guildRank = false,
		colors =
		{
			["alliance1"]= "ffff00",
         ["alliance2"] = "8B0000",
			["alliance3"] = "4169E1",
			["class1"] = "CA0000",
			["class2"] = "0096FF",
			["class3"] = "9900F1",
			["class6"] = "EFEFEF",
			["Custom1"] = "FFFFFF",	
			["Custom2"] = "FFFFFF",
			["Custom3"] = "FFFFFF",
			["Custom4"] = "FFFFFF",	
		},
		["coloring"] =
		{
			["@account"] = colorOptions[2],
			["character"] = colorOptions[2],
			["guildRank"] = colorOptions[3],
			["level"] = colorOptions[1],
		},
		characterLevel = true,
	}
}

local ChannelInfo = ZO_ChatSystem_GetChannelInfo()
local CategoryMapping = ZO_ChatSystem_GetEventCategoryMappings()

function GuildChat.GetLevel(guildName, name)
	if GuildChat.db[guildName][name].level == 50 then
      return "V" .. GuildChat.db[guildName][name].veteranRank
   end
   return GuildChat.db[guildName][name].level
end

function GuildChat.GetGuildRank(guildName, name)
	return GuildChat.db[guildName][name].rank
end

function GuildChat.GetClassColour(guildName, name)
	return GuildChat.settings.colors["class" .. GuildChat.db[guildName][name].class]
end

function GuildChat.GetAtColour(guildName, name)
	return GuildChat.settings.colors["alliance" .. GuildChat.db[guildName][name].alliance]
end

local function getSavedColor(tableEntry, name, guildName)
	if GuildChat.settings.coloring[tableEntry] == colorOptions[1] then
		if GuildChat.GetAtColour(guildName, name) then	return "|c" .. GuildChat.GetAtColour(guildName, name) end
	elseif GuildChat.settings.coloring[tableEntry] == colorOptions[2] then
		if GuildChat.GetClassColour(guildName, name) then return "|c" .. GuildChat.GetClassColour(guildName, name) end
	elseif GuildChat.settings.coloring[tableEntry] == colorOptions[3] then
		return ""
	elseif GuildChat.settings.coloring[tableEntry] == colorOptions[4] or
	       GuildChat.settings.coloring[tableEntry] == colorOptions[5] or
	       GuildChat.settings.coloring[tableEntry] == colorOptions[6] or
	       GuildChat.settings.coloring[tableEntry] == colorOptions[7] then	    
		return "|c" .. GuildChat.settings.colors[GuildChat.settings.coloring[tableEntry]]  
	end
	return ""
end

function GuildChat.GetCharacterNameLink(guildName, name)

	local characterName = GuildChat.db[guildName][name].character
	local characterLink = ""
	
	if characterName then characterLink = ZO_LinkHandler_CreatePlayerLink(characterName) end
	
	if GuildChat.settings.characterName == characterNameMenuOptions[1] then
		return ZO_LinkHandler_CreatePlayerLink(name) .. "|r"
	elseif GuildChat.settings.characterName == characterNameMenuOptions[2] then
		return characterLink
	elseif GuildChat.settings.characterName == characterNameMenuOptions[3] then
		return characterLink .. "|r" .. getSavedColor("@account", name, guildName) .. ZO_LinkHandler_CreatePlayerLink(name)
	end
	
	return ""
	
end

-- Change From Name
function GuildChat.GetName(channelId, fromName)
	
	if (channelId >= CHAT_CHANNEL_GUILD_1 and channelId <= CHAT_CHANNEL_GUILD_5) or (channelId >= CHAT_CHANNEL_OFFICER_1 and channelId <= CHAT_CHANNEL_OFFICER_5) then
	   
      local guild
		if (channelId >= CHAT_CHANNEL_GUILD_1 and channelId <= CHAT_CHANNEL_GUILD_5) then 
			guild = channelId - CHAT_CHANNEL_GUILD_1 + 1
		elseif (channelId >= CHAT_CHANNEL_OFFICER_1 and channelId <= CHAT_CHANNEL_OFFICER_5) then
			guild = channelId - CHAT_CHANNEL_OFFICER_1 + 1
		end
		
		-- Guildname
		local guildId = GetGuildId(guild)
		local guildName = GetGuildName(guildId)
		
		-- Occurs sometimes
		if(not guildName or (guildName):len() < 1) then
			guildName = "Guild " .. guild
		end
		
		if GuildChat.settings[guildName] then
			if GuildChat.settings[guildName].active then
				-- Character known?
				if not GuildChat.db[guildName][fromName] then return fromName end
				
				--d("GuildChat GetName")
				
				-- Display Character, UserID or both?
				return GuildChat.GetCharacterNameLink(guildName, fromName)
			end
		end
	end
	
	return fromName
	
end

-- Change From Name
function GuildChat.AppendInfo(channelId, fromName)
	
	if (channelId >= CHAT_CHANNEL_GUILD_1 and channelId <= CHAT_CHANNEL_GUILD_5) or (channelId >= CHAT_CHANNEL_OFFICER_1 and channelId <= CHAT_CHANNEL_OFFICER_5) then
	
		local guild = 1
		if (channelId >= CHAT_CHANNEL_GUILD_1 and channelId <= CHAT_CHANNEL_GUILD_5) then 
			guild = channelId - CHAT_CHANNEL_GUILD_1 + 1
		elseif (channelId >= CHAT_CHANNEL_OFFICER_1 and channelId <= CHAT_CHANNEL_OFFICER_5) then
			guild = channelId - CHAT_CHANNEL_OFFICER_1 + 1
		end
		
		-- Guildname
		local guildId = GetGuildId(guild)
		local guildName = GetGuildName(guildId)
		
		-- Occurs sometimes
		if(not guildName or (guildName):len() < 1) then
			guildName = "Guild " .. guild
		end
		
		if GuildChat.settings[guildName] then
			if GuildChat.settings[guildName].active then
				-- Character known?
				if not GuildChat.db[guildName][fromName] then return "" end
				
				--d("GuildChat appendInfo")
				
				local appendInfo = ""
				
				-- Display Guild Rank ?
				if GuildChat.settings.guildRank then
					if getSavedColor("guildRank", fromName, guildName) ~= "" then
						appendInfo = appendInfo .. getSavedColor("guildRank", fromName, guildName) .. "[" .. GuildChat.GetGuildRank(guildName, fromName) .. "] " .. "|r"
						GuildChat.addRtag = true
					else
						appendInfo = appendInfo .. "[" .. GuildChat.GetGuildRank(guildName, fromName) .. "] "
						GuildChat.addRtag = false
					end
				end
				-- Display Character level ?
				if GuildChat.settings.characterLevel then
					if getSavedColor("level", fromName, guildName) ~= "" then
						appendInfo = appendInfo .. getSavedColor("level", fromName, guildName) .. "[" .. GuildChat.GetLevel(guildName, fromName) .. "] " .. "|r"
						GuildChat.addRtag = true
					else
						appendInfo = appendInfo .. "[" .. GuildChat.GetLevel(guildName, fromName) .. "] "
						GuildChat.addRtag = false
					end
				end				
				
				-- Add color
				local characterName = GuildChat.db[guildName][fromName].character
				if GuildChat.settings.characterName == characterNameMenuOptions[1] then
					if getSavedColor("@account", fromName, guildName) ~= "" then
						appendInfo = appendInfo .. getSavedColor("@account", fromName, guildName)
						GuildChat.addRtag = true
					else
						GuildChat.addRtag = false
					end
				elseif GuildChat.settings.characterName == characterNameMenuOptions[2] or GuildChat.settings.characterName == characterNameMenuOptions[3] then
					if getSavedColor("character", fromName, guildName) ~= "" then
						appendInfo = appendInfo .. getSavedColor("character", fromName, guildName)
						GuildChat.addRtag = true
					else
						GuildChat.addRtag = false
					end
				end
				
				return appendInfo
			end
		end
	end
	
	return ""
	
end

-- Change From Name
function GuildChat.AppendInfoAfter(channelId, fromName)
	
	if (channelId >= CHAT_CHANNEL_GUILD_1 and channelId <= CHAT_CHANNEL_GUILD_5) or (channelId >= CHAT_CHANNEL_OFFICER_1 and channelId <= CHAT_CHANNEL_OFFICER_5) then
	
		local guild = 1
		if (channelId >= CHAT_CHANNEL_GUILD_1 and channelId <= CHAT_CHANNEL_GUILD_5) then 
			guild = channelId - CHAT_CHANNEL_GUILD_1 + 1
		elseif (channelId >= CHAT_CHANNEL_OFFICER_1 and channelId <= CHAT_CHANNEL_OFFICER_5) then
			guild = channelId - CHAT_CHANNEL_OFFICER_1 + 1
		end
		
		-- Guildname
		local guildId = GetGuildId(guild)
		local guildName = GetGuildName(guildId)
		
		-- Occurs sometimes
		if(not guildName or (guildName):len() < 1) then
			guildName = "Guild " .. guild
		end
		
		if GuildChat.settings[guildName] then
			if GuildChat.settings[guildName].active then
	
				-- Character known?
				if not GuildChat.db[guildName][fromName] then return "" end
				
				if GuildChat.addRtag then return "|r" else return "" end
				
			end
		end
	end
	
	return ""
	
end

local function RGBtoHEX(r,g,b,a)
	-- We can't have chat strings with alpha channels
	--return string.format("%.2x%.2x%.2x%.2x", math.floor(a * 255),math.floor(r * 255), math.floor(g * 255), math.floor(b * 255))	
	return string.format("%.2x%.2x%.2x", math.floor(r * 255), math.floor(g * 255), math.floor(b * 255))
end

local function HEXtoRGB(hex)
    local a, r, g, b
    
    if(string.len(hex) == 8)
    then
        a, r, g, b = tonumber("0x"..string.sub(hex, 1, 2)) / 255, tonumber("0x"..string.sub(hex, 3, 4)) / 255, tonumber("0x"..string.sub(hex, 5, 6)) / 255, tonumber("0x"..string.sub(hex, 7, 8)) / 255
    elseif(string.len(hex) == 6)
    then
        a, r, g, b = 1, tonumber("0x"..string.sub(hex, 1, 2)) / 255, tonumber("0x"..string.sub(hex, 3, 4)) / 255, tonumber("0x"..string.sub(hex, 5, 6)) / 255
    end
    
    if(a)
    then
        return r, g, b, a
    end
	
end

-- Create LAM Panel
function GuildChat.createLAM2Panel()
    local panelData = 
    {
        type = "panel",
        name = Addon.NameSpaced,
        displayName = "|cFFFFB0" .. Addon.NameSpaced .. "|r",
        author = Addon.Author,
        version = Addon.Version,
    }
  	
    local optionsData = {}	
    
    table.insert(optionsData, { type = "header", name = "Chat Options", })
    table.insert(optionsData, {	type = "dropdown", name = "Set the name to be", tooltip = "Guild Chat can display Account and/or Character name", choices = characterNameMenuOptions, getFunc = function() return GuildChat.settings.characterName end, setFunc = function(value) GuildChat.settings.characterName = value end,	disabled = function() if not (pChat or pNames) then return false else return true end end,	})
    table.insert(optionsData, { type = "checkbox", name = "Show Guild Rank", nil, getFunc = function() return GuildChat.settings.guildRank end, setFunc = function(value) GuildChat.settings.guildRank = value end, })
    table.insert(optionsData, { type = "checkbox", name = "Show Character Level", nil, getFunc = function() return GuildChat.settings.characterLevel end, setFunc = function(value) GuildChat.settings.characterLevel = value end, })
    table.insert(optionsData, { type = "header", name = "Coloring Options", })
    table.insert(optionsData, { type = "dropdown", name = "Account Name", nil, choices = colorOptions, getFunc = function() return GuildChat.settings.coloring["@account"] end, setFunc = function(value) GuildChat.settings.coloring["@account"] = value end, })
    table.insert(optionsData, { type = "dropdown", name = "Character Name", nil, choices = colorOptions, getFunc = function() return GuildChat.settings.coloring["character"] end, setFunc = function(value) GuildChat.settings.coloring["character"] = value end, })
    table.insert(optionsData, { type = "dropdown", name = "Character Level", nil, choices = colorOptions, getFunc = function() return GuildChat.settings.coloring["level"] end, setFunc = function(value) GuildChat.settings.coloring["level"] = value end, })
    table.insert(optionsData, { type = "dropdown", name = "Guild Ranking", nil, choices = colorOptions, getFunc = function() return GuildChat.settings.coloring["guildRank"] end, setFunc = function(value) GuildChat.settings.coloring["guildRank"] = value end, })    
    table.insert(optionsData, { type = "header", name = "Faction Colors", })
    table.insert(optionsData, { type = "colorpicker", name = zo_strformat(SI_ALLIANCE_NAME, GetAllianceName(1)), getFunc = function() return HEXtoRGB(GuildChat.settings.colors["alliance1"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["alliance1"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "colorpicker", name = zo_strformat(SI_ALLIANCE_NAME, GetAllianceName(2)), getFunc = function() return HEXtoRGB(GuildChat.settings.colors["alliance2"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["alliance2"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "colorpicker", name = zo_strformat(SI_ALLIANCE_NAME, GetAllianceName(3)), getFunc = function() return HEXtoRGB(GuildChat.settings.colors["alliance3"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["alliance3"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "header", name = "Class Colors", })
    table.insert(optionsData, { type = "colorpicker", name = zo_strformat(SI_CLASS_NAME, GetClassName(0, 1)), getFunc = function() return HEXtoRGB(GuildChat.settings.colors["class1"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["class1"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "colorpicker", name = zo_strformat(SI_CLASS_NAME, GetClassName(0, 2)), getFunc = function() return HEXtoRGB(GuildChat.settings.colors["class2"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["class2"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "colorpicker", name = zo_strformat(SI_CLASS_NAME, GetClassName(0, 3)), getFunc = function() return HEXtoRGB(GuildChat.settings.colors["class3"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["class3"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "colorpicker", name = zo_strformat(SI_CLASS_NAME, GetClassName(0, 6)), getFunc = function() return HEXtoRGB(GuildChat.settings.colors["class6"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["class6"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "header", name = "Custom Colors", })
    table.insert(optionsData, { type = "colorpicker", name = "Custom #1", tooltip = "Custom #1 Color.", getFunc = function() return HEXtoRGB(GuildChat.settings.colors["Custom1"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["Custom1"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "colorpicker", name = "Custom #2", tooltip = "Custom #2 Color.", getFunc = function() return HEXtoRGB(GuildChat.settings.colors["Custom2"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["Custom2"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "colorpicker", name = "Custom #3", tooltip = "Custom #3 Color.", getFunc = function() return HEXtoRGB(GuildChat.settings.colors["Custom3"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["Custom3"] = RGBtoHEX(r,g,b,a) end, })
    table.insert(optionsData, { type = "colorpicker", name = "Custom #4", tooltip = "Custom #4 Color.", getFunc = function() return HEXtoRGB(GuildChat.settings.colors["Custom4"]) end, setFunc = function(r,g,b,a) GuildChat.settings.colors["Custom4"] = RGBtoHEX(r,g,b,a) end, })
     
	for guild = 1, GetNumGuilds() do
		
		-- Guildname
		local guildId = GetGuildId(guild)
		local guildName = GetGuildName(guildId)
		
		-- Occurs sometimes
		if(not guildName or (guildName):len() < 1) then
			guildName = "Guild " .. guild
		end
		
		-- 1st launch
		if not GuildChat.settings[guildName] then
			GuildChat.settings[guildName] = {}
			GuildChat.settings[guildName].active = true
		end
		
		table.insert(optionsData, {	type = "header", name = guildName, width = "full",})
		table.insert(optionsData, { type = "checkbox", name = "Activate Guild Chat", tooltip = "Activate Guild Chat for this guild", getFunc = function() return GuildChat.settings[guildName].active end, setFunc = function(newValue) GuildChat.settings[guildName].active = newValue end, width = "full",})
		
	end	 
	
   	LAM2:RegisterAddonPanel(Addon.Name.."LAM2Options", panelData)    
    LAM2:RegisterOptionControls(Addon.Name.."LAM2Options", optionsData)
	
end

-- Register with LibChat
function GuildChat.Register()

	-- Register only a single time
	if GuildChat.onlyAt1stActivation then
   
		LC:registerAppendTextBeforeSender(GuildChat.AppendInfo, Addon.Name)
		LC:registerAppendTextAfterSender(GuildChat.AppendInfoAfter, Addon.Name)
		
		-- Don't do this if others addons are loaded
		if not (pChat or pNames) then
			--d("GuildChat registering Name")
			LC:registerName(GuildChat.GetName, Addon.Name)
		end
      
      for guild = 1, GetNumGuilds() do
         local guildId = GetGuildId(guild)
         GuildChat.createTable(guildId)
      end
		
		EVENT_MANAGER:UnregisterForEvent(Addon.Name, EVENT_PLAYER_ACTIVATED)
		GuildChat.onlyAt1stActivation = false
      
	end
	
end

function GuildChat.createTable(guildId, DisplayName)
	
   local guildName = GetGuildName(guildId)
   
   if(not guildName or (guildName):len() < 1) then
      guildName = "Guild " .. guildId
   end
   
   if not DisplayName then
      GuildChat.db[guildName] = {}
	
      -- Iterate over each guild member
      for member = 1, GetNumGuildMembers(guildId) do
         
         local account, _, rankIndex, playerStatus = GetGuildMemberInfo(guildId, member)
         local hasChar, character, zoneName, classType, alliance, level, veteranRank = GetGuildMemberCharacterInfo(guildId, member)
         local rankName = GetGuildRankCustomName(guildId, rankIndex)
         if rankName == "" then rankName = GetFinalGuildRankName(guildId, rankIndex) end
         
         if hasChar and character then
            
            -- Remove extra characters.
            character = zo_strformat(SI_UNIT_NAME, character)
            GuildChat.db[guildName][account] = {
               character = character,
               rank = rankName,
               status = playerStatus,
               class = classType,
               alliance = alliance,
               level = level,
               veteranRank = veteranRank,
            }
            
         end
      end
	else
   
		local member = GetGuildMemberIndexFromDisplayName(guildId, DisplayName)
		local account, _, rankIndex, playerStatus = GetGuildMemberInfo(guildId, member)
		local rankName = GetGuildRankCustomName(guildId, rankIndex)
		
		if rankName == "" then rankName = GetFinalGuildRankName(guildId, rankIndex) end
		local hasChar, character, zoneName, classType, alliance, level, veteranRank = GetGuildMemberCharacterInfo(guildId, member)
		
		if hasChar and character then
			
			-- Remove extra characters.
			character = zo_strformat(SI_UNIT_NAME, character)
			
			if GuildChat.db[guildName] and GuildChat.db[guildName][account] then
				GuildChat.db[guildName][account] = {
					character = character,
					rank = rankName,
					status = playerStatus,
					class = classType,
					alliance = alliance,
					level = level,
					veteranRank = veteranRank,
				}
			end
			
		end
   end
   
end

function GuildChat.OnGuildMemberLevelUpdated(_, guildId, DisplayName, _, newLevel)

   local guildName = GetGuildName(guildId)
   if GuildChat.db[guildName] then
      if GuildChat.db[guildName][DisplayName] then
         GuildChat.db[guildName][DisplayName].level = newLevel
      end
   end
	
end

function GuildChat.OnGuildMemberVetRankUpdated(_, guildId, DisplayName, _, newVeteranRank)

   local guildName = GetGuildName(guildId)
   if GuildChat.db[guildName] then
      if GuildChat.db[guildName][DisplayName] then
         GuildChat.db[guildName][DisplayName].veteranRank = newVeteranRank
      end
   end
	
end

function GuildChat.OnGuildMemberRankUpdated(_, guildId, DisplayName, newRank)

   local guildName = GetGuildName(guildId)
   if GuildChat.db[guildName] then
      if GuildChat.db[guildName][DisplayName] then
         GuildChat.db[guildName][DisplayName].rank = newRank
      end
   end
	
end

function GuildChat.OnGuildMemberAdded(_, guildId, DisplayName)
   GuildChat.createTable(guildId, DisplayName)
end

function GuildChat.OnGuildMemberUpdate(_, guildId, DisplayName, oldStatus, newStatus)
   if newStatus ~= PLAYER_STATUS_OFFLINE then
      GuildChat.createTable(guildId, DisplayName)
   end
end

function GuildChat.OnGuildJoined(_, guildId, guildName)
   GuildChat.createTable(guildId)
end

function GuildChat.OnGuildLeft(_, _, guildName)
   GuildChat.db[guildName] = nil
end


-- Load Addon into Memory
function GuildChat.onAddonLoaded(_, addonName)
	
	-- Protect
	if addonName ~= Addon.Name then return end
	
	-- Load settings
	GuildChat.settings = ZO_SavedVars:NewAccountWide('GuildChat_Settings', 4, nil, Addon.Defaults)
	
	-- Build LAM
	GuildChat.createLAM2Panel()
	
	GuildChat.onlyAt1stActivation = true
	
	-- Unregisters
	EVENT_MANAGER:UnregisterForEvent(Addon.Name, EVENT_ADD_ON_LOADED)
	
end

-- Register events
EVENT_MANAGER:RegisterForEvent(Addon.Name, EVENT_ADD_ON_LOADED, GuildChat.onAddonLoaded)
EVENT_MANAGER:RegisterForEvent(Addon.Name, EVENT_PLAYER_ACTIVATED, GuildChat.Register)
EVENT_MANAGER:RegisterForEvent(Addon.Name, EVENT_GUILD_SELF_JOINED_GUILD, GuildChat.OnGuildJoined)
EVENT_MANAGER:RegisterForEvent(Addon.Name, EVENT_GUILD_SELF_LEFT_GUILD, GuildChat.OnGuildLeft)
EVENT_MANAGER:RegisterForEvent(Addon.Name, EVENT_GUILD_MEMBER_ADDED, GuildChat.OnGuildMemberAdded)
EVENT_MANAGER:RegisterForEvent(Addon.Name, EVENT_GUILD_MEMBER_CHARACTER_LEVEL_CHANGED, GuildChat.OnGuildMemberLevelUpdated)
EVENT_MANAGER:RegisterForEvent(Addon.Name, EVENT_GUILD_MEMBER_CHARACTER_VETERAN_RANK_CHANGED, GuildChat.OnGuildMemberVetRankUpdated)
EVENT_MANAGER:RegisterForEvent(Addon.Name, EVENT_GUILD_MEMBER_RANK_CHANGED, GuildChat.OnGuildMemberRankUpdated)
EVENT_MANAGER:RegisterForEvent(Addon.name, EVENT_GUILD_MEMBER_PLAYER_STATUS_CHANGED, GuildChat.OnGuildMemberUpdate)